from is_domain_free.base import are_free
from is_domain_free.base import DomainNameAvailabilityChecker
from is_domain_free.base import get_tld_from_domain_name
from is_domain_free.base import is_free
from is_domain_free.base import WhoisGetterBasic
from is_domain_free.base import WhoisParserBasic



from pathlib import Path
from unittest import TestCase

import os

from is_domain_free import DomainNameAvailabilityChecker
from is_domain_free import get_tld_from_domain_name
from is_domain_free import WhoisParserBasic
from is_domain_free.exceptions import TldNotSupportedException


def get_domain_file_content(domain_name):
    """
    Simple utility for tests used to read a text file from a domain name
    :param domain_name: Domain name for which to read the corresponding file
    :return: Content of the text file
    """
    content = Path('%s/whois_results/%s.txt' % (os.path.dirname(__file__), domain_name)).read_text()
    return content


class WhoisContentGetterStub(object):
    """
    Whois content getter used for tests. It will get the
    text form the text file based on the domain name
    passed in parameter.

    For example, if you pass 'google.io', it will look for
    google.io.txt in the 'whois_results' folder.
    """

    def get_whois_content(self, domain):
        content = get_domain_file_content(domain)
        return content


class WhoisParserBasicTest(TestCase):
    """
    Tests that the basic whois parser returns the right availability
    statuses depending on string found in whois content
    """
    def _is_available(self, domain_name, whois_content):
        """
        Checks if a domain_name is available based on the
        content of a whois command

        :param domain_name: domain name to check
        :param whois_content: Text (content of a whois command)
        :return:
        """
        parser = WhoisParserBasic()
        is_available = parser.is_available(domain_name, whois_content)
        return is_available

    def test_parser_tld_com(self):
        """
        Tests that the parser stub finds the content of a file
        :return:
        """
        tld = get_tld_from_domain_name('google.com')
        self.assertTrue(tld, 'com')

    def test_get_tld_uk(self):
        """
        Tests that a tld from a .co.uk returns .uk
        :return:
        """
        tld = get_tld_from_domain_name('google.co.uk')
        self.assertTrue(tld, 'uk')

    def test_unsupported_tld(self):
        """
        Tests that an error is raised when an unsupported TLD is passed
        :return:
        """
        parser = WhoisParserBasic()
        self.assertRaises(TldNotSupportedException, parser.is_available, 'unsupportedtld.abc', 'NOT FOUND')

    def test_parse_com_unavail(self):
        """
        Test that we successfully say that a domain name is
        taken/not available when it's registered eg google.com
        """
        self.assertFalse(self._is_available('google.com', 'XXX'))

    def test_parse_com_avail(self):
        """
        Tests that the result of a .com returns that
        the domain is available when nobody registered it.
        """
        self.assertTrue(self._is_available('rewqwerty.com', WhoisParserBasic.COM_NET_AVAILABLE))

    def test_parse_net_unavail(self):
        """
        unavailability test for a .net domain
        """
        self.assertFalse(self._is_available('google.net', 'XXX'))

    def test_parse_net_avail(self):
        """
        availability test for a .net domain
        """
        self.assertTrue(self._is_available('rewqwerty.net', WhoisParserBasic.COM_NET_AVAILABLE))

    def test_parse_io_unavail(self):
        """
        Tests that if the specific text isn't find in the whois content,
        the domain name is returned as being taken.
        :return:
        """
        self.assertFalse(self._is_available('google.io', 'XXX'))

    def test_parse_io_avail(self):
        """
        Tests that a domain name is available if it contains the
        searched string indicating that a domain is not taken.
        :return:
        """
        self.assertTrue(self._is_available('rewqwerty.io', WhoisParserBasic.IO_AVAILABLE))


class WhoisContentGetterStubTest(TestCase):
    """
    Simply testing that the stub returns text file content
    """
    def test_get_file(self):
        """
        Testing that when getting the content of google.com, it reads
        the google.com.txt file and returns its content.
        :return:
        """
        whois_stub = WhoisContentGetterStub()
        content = whois_stub.get_whois_content('google.com')
        self.assertTrue(content.startswith('   Domain Name: GOOGLE.COM'))


class IntegrationTest(TestCase):
    """
    Test using both the whois getter and the whois parser.
    """
    def _get_domain_checker(self):
        whois_getter = WhoisContentGetterStub()
        whois_parser = WhoisParserBasic()

        domain_checker = DomainNameAvailabilityChecker(whois_getter, whois_parser)
        return domain_checker

    def _is_available(self, domain_name):
        domain_checker = self._get_domain_checker()
        return domain_checker.is_available(domain_name)

    def test_whois_getter(self):
        whois_getter = WhoisContentGetterStub()
        file_content = whois_getter.get_whois_content('google.io')
        self.assertTrue('Domain Name: GOOGLE.IO' in file_content)

    def test_available_domain_com(self):
        self.assertTrue(self._is_available('rewqwerty.com'))

    def test_unavailable_domain_com(self):
        self.assertFalse(self._is_available('google.com'))

    def test_available_domain_net(self):
        self.assertTrue(self._is_available('rewqwerty.net'))

    def test_unavailable_domain_net(self):
        self.assertFalse(self._is_available('google.net'))

    def test_available_domain_io(self):
        self.assertTrue(self._is_available('rewqwerty.io'))

    def test_unavailable_domain_io(self):
        self.assertFalse(self._is_available('google.io'))

    def test_multiple_domains(self):
        """
        Testing a batch of domain names for their availability

        :return:
        """
        domain_names = [
            'google.com',
            'rewqwerty.io',
            'google.io',
        ]

        domain_checker = self._get_domain_checker()

        result = domain_checker.is_available_batch(domain_names)

        self.assertEqual(len(result), 3)
        self.assertListEqual(sorted(result), sorted([
            ('google.com', False),
            ('rewqwerty.io', True),
            ('google.io', False),
        ]))



import multiprocessing as mp
import subprocess
import sys

from .exceptions import TldNotSupportedException


name = "is_domain_free"


def get_tld_from_domain_name(domain_name):
    """
    :param domain_name: the domain name from which to get the TLD from
    :return: Returns the TLD from a domain name (google.com would return 'com')
    """
    return domain_name.split('.')[-1]


class DomainNameAvailabilityChecker(object):
    """
    Main object that checks the availability of a domain name.

    The whois getter is the object that will return the content of a
    whois command on a specific domain name.

    The whois parser is the object that will parse the whois
    content.
    """
    def __init__(self, whois_getter, whois_parser):
        """
        :param whois_getter: Object that gets the whois content
        :param whois_parser: Object that parses the whois content
        """
        self._whois_getter = whois_getter
        self._whois_parser = whois_parser

    def is_available(self, domain_name):
        """
        Determine if a domain name is available (not taken) or
        unavailable (taken).

        :param domain_name: The domain name to check
        :return: True if the domain name is available (not taken)
        """
        whois_content = self._whois_getter.get_whois_content(domain_name)
        return self._whois_parser.is_available(domain_name, whois_content)

    def _is_available_batch(self, domain_name, output):
        """
        Used internally with is_available batch running parallel
        tasks. Output is used to append results.

        :param domain_name: domain name to check
        :param output: Results are added in output
        :return:
        """
        return output.put((domain_name, self.is_available(domain_name)))

    def is_available_batch(self, domain_names):
        """
        Multiple domain names checking, running in parellel. Since there's
        a lot of wait time with the 'whois' command, 100 domain name
        checks can take 10 seconds if ran sequentially. It goes down to
        about 1 second if we run the code with parallel processes.

        :param domain_names: Iterable conaintain the list of domains to
        check

        :return: A list of tuples with the domain names and a boolean
          indicating if they are available or not.
        """
        output = mp.Queue()
        processes = [
            mp.Process(
                target=self._is_available_batch, args=(domain_name, output)
            ) for domain_name in domain_names
        ]

        for p in processes:
            p.start()

        for p in processes:
            p.join()

        results = [output.get() for p in processes]
        return results


class WhoisParserBasic(object):
    """
    Basic WHOIS content result parser. It simply checks
    in the whois content if a specific string appears at the
    beginning for .com, .net and .io domain names.
    """

    COM = 'com'
    NET = 'net'
    IO = 'io'

    supported_tlds = (COM, NET, IO)

    # String that means a domain name is available
    COM_NET_AVAILABLE = "No match for \""
    IO_AVAILABLE = "NOT FOUND"

    def is_available(self, domain_name, whois_content):
        if not whois_content:
            raise ValueError("whois_content parameter shouldn't be empty")

        tld = get_tld_from_domain_name(domain_name)

        if tld not in self.supported_tlds:
            raise TldNotSupportedException("tld must be one of: %s" % str(self.supported_tlds))

        if tld == self.COM or tld == self.NET:
            if whois_content.startswith(WhoisParserBasic.COM_NET_AVAILABLE):
                return True

        if tld == self.IO:
            if whois_content.startswith(WhoisParserBasic.IO_AVAILABLE):
                return True

        return False


class WhoisGetterBasic(object):
    """
    Simple basic whois content getter from the command line
    running whois abc.com. It will grab the whole text
    and return it back.
    """

    def get_whois_content(self, domain):
        """
        Returns the content of a whois 'domain-name.com' command

        Will simply run the 'whois' command on the current system and return
        the string (based on default local encoding).

        :param domain: Domain to check
        :return: The content of the whois command
        """
        result = subprocess.run(['whois', domain], stdout=subprocess.PIPE)
        return result.stdout.decode(sys.getdefaultencoding())


def is_free(domain_name):
    """
    Simple shortcut command that will instantiate a DomainNameAvailabilityChecker
     with the basic whois content grabber and whois parser.

    :param domain_name: Single domain name to check
    :return: List of tuples ie [(domainabc.com, True), ...]
     indicating if the domain names are free or not.
    """
    checker = DomainNameAvailabilityChecker(WhoisGetterBasic(), WhoisParserBasic())
    return checker.is_available(domain_name)


def are_free(domain_names):
    """
    Simple shortcut command that will instantiate a DomainNameAvailabilityChecker
     with the basic whois content grabber and whois parser and check a list
     of domain names for their availability.

    :param domain_names: List of domain names to check
    :return: List of tuples ie [(domainabc.com, True), ...]
     indicating if the domain names are free or not.
    """
    checker = DomainNameAvailabilityChecker(WhoisGetterBasic(), WhoisParserBasic())
    return checker.is_available_batch(domain_names)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="is_domain_free",
    version="0.0.1",
    author="Marc-Olivier Gosselin",
    author_email="mogosselin@gmail.com",
    description="Simple package to check if 1 or more domain names are available.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mogosselin/is_domain_free",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
